@@ -1,5 +1,5 @@
 ## File Name: frm_modify_parameter_labels.R
-## File Version: 0.06
+## File Version: 0.11
 
 frm_modify_parameter_labels <- function( dfr, ind0, NM )
 {
@@ -12,10 +12,14 @@ frm_modify_parameter_labels <- function( dfr, ind0, NM )
         if ( ind0[[mm]]$model %in% c("bctreg", "yjtreg")){
             indices_mm <- which( dfr$model==mm )
             dfr1 <- dfr[ indices_mm, ]
-            NR <- nrow(dfr1)
-            index_mm <- seq(NR-1,NR)
-            dfr1[ index_mm, "ON"] <- 0
-            dfr1[ index_mm, "parm"] <-  paste0( dv_mm, c(" sigma", " lambda" ) )
+            dfr1$parm <- gsub( "ON sigma", "sigma", dfr1$parm )
+            dfr1$parm <- gsub( "ON lambda", "lambda", dfr1$parm )
+            dfr1$parm <- gsub( "ON logdf", "logdf", dfr1$parm )
+            ind_fill <- grep( " ON ", dfr1$parm)
+            if ( length(ind_fill) > 0 ){
+                dfr1$ON <- 0
+                dfr1[ind_fill, "ON"] <- 1
+            }
             dfr[ indices_mm, ] <- dfr1
         }
     }
